using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using QueryApplication.Complete.DataWebService;
using System.Linq.Expressions;
using System.Reflection;

namespace QueryApplication.Complete {
	public class OrderingEnabledDataWebServiceQuery : IOrderedQueryable<CountryInfo>, IQueryProvider {
		public OrderingEnabledDataWebServiceQuery(Service dataWebService) {
			this.dataWebService = dataWebService;
		}

		Service dataWebService;
		Expression expression;

		#region Important methods
		#region IQueryable.Provider

		IQueryProvider IQueryable.Provider {
			get {
				//Console.WriteLine( "In IQueryable.Provider ");

				return this;
			}
		}

		#endregion
		#region IQueryable.Expression
		Expression IQueryable.Expression {
			get {
				//Console.WriteLine("In IQueryable.Expression");

				return expression ?? Expression.Constant(this);
			}
		}
		#endregion
		#region IQueryProvider.CreateQuery<TResult>
		IQueryable<TResult> IQueryProvider.CreateQuery<TResult>(Expression expression) {
			//Console.WriteLine( "In IQueryProvider.CreateQuery<TResult>");
			//ExpressionDumper.Output(expression);
			//Console.ReadLine( );

			this.expression = expression;
			return (IQueryable<TResult>) this;
		}
		#endregion
		#region IEnumerable<CountryInfo>.GetEnumerator

		IEnumerator<CountryInfo> IEnumerable<CountryInfo>.GetEnumerator( ) {
			//Console.WriteLine( "In IEnumerable<CountryInfo>.GetEnumerator");

			SearchInfo searchInfo = ParseExpression(expression);
			CountryInfo[] result = dataWebService.GetFilteredAndSortedData(searchInfo.FilterPrefix, searchInfo.SortOrder);
			return ((IEnumerable<CountryInfo>) result).GetEnumerator( );
		}
		#endregion


		#endregion

		// My own method, to interpret the expression
		#region ParseExpression
		SearchInfo ParseExpression(Expression expression) {
			//Console.WriteLine( "In ParseExpression");
			//ExpressionDumper.Output(expression);
			//Console.ReadLine( );

			SearchInfo result = new SearchInfo( );

			if (expression.NodeType != ExpressionType.Call)
				throw new NotSupportedException("Only Call expressions supported.");

			MethodCallExpression orderbyMethodCall = (MethodCallExpression) expression;
			if (orderbyMethodCall.Method.Name != "OrderBy")
				throw new NotSupportedException("Only OrderBy MethodCalls supported.");

			if (orderbyMethodCall.Arguments.Count != 2)
				throw new NotSupportedException("Only argument count 2 supported.");

			if (orderbyMethodCall.Arguments[1].NodeType != ExpressionType.Quote)
				throw new NotSupportedException("Only Quote supported here.");

			UnaryExpression orderbyArgumentQuote = ((UnaryExpression) orderbyMethodCall.Arguments[1]);

			if (orderbyArgumentQuote.Operand.NodeType != ExpressionType.Lambda)
				throw new NotSupportedException("Only OrderBy argument type Lambda supported.");

			LambdaExpression orderbyLambda = ((LambdaExpression) orderbyArgumentQuote.Operand);

			if (orderbyLambda.Body.NodeType != ExpressionType.MemberAccess)
				throw new NotSupportedException("Only MemberAccess expressions supported for lambda body.");

			MemberExpression memberExpression = (MemberExpression) orderbyLambda.Body;
			if (memberExpression.Member.DeclaringType.Name != "CountryInfo")
				throw new NotSupportedException("Only members of class CountryInfo supported.");
			if (memberExpression.Member.MemberType != MemberTypes.Property)
				throw new NotSupportedException("Only members of type Property supported.");
			switch (memberExpression.Member.Name) {
				case "Name":
					result.SortOrder = DataSortOrder.Name;
					break;
				case "Population":
					result.SortOrder = DataSortOrder.Population;
					break;
				default:
					throw new NotSupportedException("Only members named 'Name' or 'Population' supported.");
			}

			MethodCallExpression whereMethodCall = (MethodCallExpression)
				((MethodCallExpression) expression).Arguments[0];

			if (whereMethodCall.Method.Name != "Where")
				throw new NotSupportedException("Only Where calls supported.");

			if (whereMethodCall.Arguments.Count != 2)
				throw new NotSupportedException("Only argument count 2 supported.");

			if (whereMethodCall.Arguments[1].NodeType != ExpressionType.Quote)
				throw new NotSupportedException("Only Quote supported here.");

			UnaryExpression whereArgumentQuote = ((UnaryExpression) whereMethodCall.Arguments[1]);

			if (whereArgumentQuote.Operand.NodeType != ExpressionType.Lambda)
				throw new NotSupportedException("Only Where argument type Lambda supported.");

			LambdaExpression whereArgumentLambda = ((LambdaExpression) whereArgumentQuote.Operand);

			if (whereArgumentLambda.Body.NodeType != ExpressionType.Call)
				throw new NotSupportedException("Only Call expressions supported for lambda body.");

			MethodCallExpression whereArgumentLambdaMethodCall = (MethodCallExpression) whereArgumentLambda.Body;
			if (whereArgumentLambdaMethodCall.Method.Name != "StartsWith")
				throw new NotSupportedException("Only lambda body calls to string.StartsWith supported.");

			ConstantExpression startsWithString = (ConstantExpression) whereArgumentLambdaMethodCall.Arguments[0];
			if (startsWithString.Type != typeof(string))
				throw new NotSupportedException("Only string arguments to string.StartsWith supported.");

			result.FilterPrefix = (string) startsWithString.Value;

			return result;
		}
		#endregion

		#region Other stuff
		#region IQueryable.ElementType
		Type IQueryable.ElementType {
			get {
				throw new NotImplementedException( );
			}
		}
		#endregion
		
		#region IEnumerable.GetEnumerator

		IEnumerator IEnumerable.GetEnumerator( ) {
			throw new NotImplementedException( );
		}

		#endregion
		#region IQueryProvider Members


		IQueryable IQueryProvider.CreateQuery(Expression expression) {
			throw new NotImplementedException( );
		}

		TResult IQueryProvider.Execute<TResult>(Expression expression) {
			throw new NotImplementedException( );
		}

		object IQueryProvider.Execute(Expression expression) {
			throw new NotImplementedException( );
		}

		#endregion
		#endregion

	}

	public class SearchInfo {
		public SearchInfo( ) {

		}

		public SearchInfo(string filterPrefix, DataSortOrder sortOrder) {
			this.filterPrefix = filterPrefix;
			this.sortOrder = sortOrder;
		}

		private string filterPrefix;
		public string FilterPrefix {
			get {
				return filterPrefix;
			}
			set {
				filterPrefix = value;
			}
		}

		private DataSortOrder sortOrder;
		public DataSortOrder SortOrder {
			get {
				return sortOrder;
			}
			set {
				sortOrder = value;
			}
		}
	}
}
