﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using QueryApplication.Complete.DataWebService;

namespace QueryApplication.Complete {
	class Program {
		static void Main(string[] args) {
			var dataWebService = new Service( );

			//ShowAllData(dataWebService);
			//ShowFilteredData(dataWebService);
			//ShowLINQFilteredData(dataWebService);
			ShowLINQThroughAdapterData(dataWebService);
			//ShowLINQThroughAdapterOrderedData(dataWebService);

			Console.ReadLine( );
		}

		private static void ShowAllData(Service dataWebService) {
			var list = dataWebService.GetData( );
			foreach (CountryInfo countryInfo in list) {
				Console.WriteLine(
					String.Format("Country: {0}, Population: {1}, Area: {2}",
					countryInfo.Name, countryInfo.Population, countryInfo.AreaKM2));
			}
		}

		private static void ShowFilteredData(Service dataWebService) {
			var list = dataWebService.GetFilteredData("T");

			foreach (CountryInfo countryInfo in list) {
				Console.WriteLine(
					String.Format("Country: {0}, Population: {1}, Area: {2}",
					countryInfo.Name, countryInfo.Population, countryInfo.AreaKM2));
			}
		}

		private static void ShowLINQFilteredData(Service dataWebService) {
			var list =
							from entry in dataWebService.GetData( )
							where entry.Name.StartsWith("T")
							select entry;

			foreach (CountryInfo countryInfo in list) {
				Console.WriteLine(
					String.Format("Country: {0}, Population: {1}, Area: {2}",
					countryInfo.Name, countryInfo.Population, countryInfo.AreaKM2));
			}
		}

		private static void ShowLINQThroughAdapterData(Service dataWebService) {
			var list =
							from entry in dataWebService.GetQuery()
							where entry.Name.StartsWith("T")
							select entry;

			foreach (CountryInfo countryInfo in list) {
				Console.WriteLine(
					String.Format("Country: {0}, Population: {1}, Area: {2}",
					countryInfo.Name, countryInfo.Population, countryInfo.AreaKM2));
			}
		}

		private static void ShowLINQThroughAdapterOrderedData(Service dataWebService) {
			var list =
							from entry in dataWebService.GetOrderableQuery()
							where entry.Name.StartsWith("T")
							orderby entry.Population
							select entry;

			foreach (CountryInfo countryInfo in list) {
				Console.WriteLine(
					String.Format("Country: {0}, Population: {1}, Area: {2}",
					countryInfo.Name, countryInfo.Population, countryInfo.AreaKM2));
			}
		}
	}

	public static class QueryingHelpers {
		public static DataWebServiceQuery GetQuery(this Service service) {
			return new DataWebServiceQuery(service);
		}

		public static OrderingEnabledDataWebServiceQuery GetOrderableQuery(this Service service) {
			return new OrderingEnabledDataWebServiceQuery(service);
		}
	}
}
